use network_types::ip::IpProto;

use crate::elk::elasticsearch::ElasticConection;

use super::ip_group::ConectionCoreAtom;
use std::{
    future::IntoFuture,
    net::Ipv6Addr,
    time::{Duration, SystemTime, UNIX_EPOCH},
};

pub struct net_stats_storage {
    connections_store: ConectionCoreAtom,
    elastic_connection: ElasticConection,
    old_connections: Vec<ConectionCoreAtom>,
    last_insert: SystemTime,
}

impl net_stats_storage {
    pub fn new() -> Self {
        Self {
            connections_store: ConectionCoreAtom::new(),
            elastic_connection: ElasticConection::new().unwrap(),
            old_connections: Vec::new(),
            last_insert: SystemTime::now(),
        }
    }

    pub async fn addv4(
        &mut self,
        source_ip: u32,
        dest_ip: u32,
        source_port: u16,
        dest_port: u16,
        protocol: IpProto,
        size: u16,
    ) {
        self.store_or_not().await;

        self.connections_store
            .addv4(source_ip, dest_ip, source_port, dest_port, protocol, size);
    }

    pub async fn addv6(
        &mut self,
        source_ip: Ipv6Addr,
        dest_ip: Ipv6Addr,
        source_port: u16,
        dest_port: u16,
        protocol: IpProto,
        size: u16,
    ) {
        self.store_or_not().await;

        self.connections_store
            .addv6(source_ip, dest_ip, source_port, dest_port, protocol, size);
    }

    pub async fn store_or_not(&mut self) {
        if self
            .last_insert
            .duration_since(SystemTime::now())
            .unwrap_or(Duration::from_secs(2))
            .as_secs()
            > 2
            || self.connections_store.len > 200000
        {
            self.store_in_elastic().await;
        }
    }

    pub async fn store_in_elastic(&mut self) {
        self.old_connections.push(self.connections_store.reset());
        self.connections_store = ConectionCoreAtom::new();
        println!("entra:{}", self.old_connections.len());
        for con in &self.old_connections {
            println!(
                "datos:{}",
                chrono::offset::Utc::now().to_rfc3339_opts(chrono::SecondsFormat::Secs, true)
            );
            match self
                .elastic_connection
                .send(con.generate_json(
                    chrono::offset::Utc::now().to_rfc3339_opts(chrono::SecondsFormat::Secs, true),
                ))
                .await
            {
                Ok(_) => continue,
                Err(_) => {
                    println!("No va");
                    break;
                }
            }
        }
    }
}
