use elasticsearch::auth::Credentials;
use elasticsearch::http::transport::SingleNodeConnectionPool;
use elasticsearch::http::transport::TransportBuilder;
use elasticsearch::http::Url;
use elasticsearch::Elasticsearch;
use elasticsearch::IndexParts;
use serde_json::Value;
use std::fmt;

pub struct ElasticConection {
    conection: Elasticsearch,
}

pub struct ElasticConErr {
    content: String,
}

impl ElasticConection {
    pub fn new() -> Result<Self, Box<dyn std::error::Error>> {
        let url = Url::parse("http://elastic_ip:9200")?;
        let credentials = Credentials::Basic("elastic".into(), "password".into());
        let connection_pool = SingleNodeConnectionPool::new(url);
        let transport = TransportBuilder::new(connection_pool)
            .auth(credentials)
            .build()?;
        Ok(Self {
            conection: Elasticsearch::new(transport),
        })
    }

    pub async fn send(&self, data: Value) -> Result<(), ElasticConErr> {
        println!("aquí tambi'en entra");
        let raw_response = self
            .conection
            .index(IndexParts::Index("netlogger-0.1"))
            .body(data)
            .send()
            .await;
        let response = match raw_response {
            Ok(r) => r,
            Err(e) => {
                return Err(ElasticConErr {
                    content: e.to_string(),
                })
            }
        };
        let status_code = response.status_code();
        match status_code.clone().is_success() {
            true => Ok(()),
            false => {
                let err_ret = match response.text().await {
                    Ok(ret) => ret,
                    Err(_e) => status_code.as_str().to_string(),
                };
                Err(ElasticConErr { content: err_ret })
            }
        }
    }
}

impl fmt::Display for ElasticConErr {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "{}", self.content)
    }
}

impl fmt::Debug for ElasticConErr {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "{}", self.content)
    }
}
